#!/usr/bin/perl -w

use strict;
use Carp;
use RRDs; # available with rrdtool, type 'make site-perl-install'
use Net::SNMP; #available at CPAN
use vars qw($session $error $response);
use Sys::Syslog; # log to syslog
use Time::localtime; 
my %general; # general options
my %routers; # routers + communityes
my %networks; # for ip accounting, wich will be available next releases
my $graphdir; # directory for graphics
my $logdir; # directory for rrd files
my ($community,$hostname);
my $configfile = '/etc/cimon.conf';#edit here if needed

#Parse config

open (CONF,"$configfile") || croak("Cannot open $!");
openlog('cimon', 'cons,pid,nowait', 'user');
syslog('info', 'cimon started');

while(<CONF>){
	  s/^#\w+|\s//g; #remove comments
	  #s/^\s+\t+//;   #remove blank lines
	  s/^\s+|\s+$//g;
	  if(/general\{|^\{/ .. /\}|\}$/){	
		 s/\}$//g;
		 if (/^graphdir/){
	 		s/graphdir\=//g;		
			s/\/$//g;
			$_ =  $_."\/";
			$graphdir = $_;
		 }
		 if ($_ =~ /^logdir/){
			s/logdir\=//g;
                        s/\/$//g;$_ = $_."\/";
                        $logdir = $_;
		 } else { 
			$logdir = "/var/cimon/";
			}
	  }
	  if(/router\{|^\{/ .. /\}|\}$/){
		s/\}$//g;
		if (/^hostname/){
			(my $j,my $i)=split(/=/,$_);
			($community,$hostname)=split(/@/,$i);
			$routers{$hostname}= $community;
		}
		if (/^network\=/){
			(my $j,my $i)= split(/=/,$_);
			my @hosts = split(/\,/,$i);
			$networks{$hostname}=\@hosts;
		}
	}
}
close CONF;

#Degub config parse
#foreach(%routers){print $_,"\n";}
#foreach(values %networks) {print @$_,"\n";}

#some directory checking

if (!opendir(DIR,$graphdir)){
	syslog('warning',"WARNING graphdir options not found");
	croak("Supply a directory where graphics will be located");
}
if (!opendir(DIR,$logdir)){
	mkdir("/var/cimon",0755) && syslog('info',"/var/cimon created") ||
	croak("Neither logdir found nor /var/cimon created 
	Maybe su root?");
	
}
foreach(keys %routers) {
	&get_info($_,$routers{$_});
	&graph($_);
}
closelog();# syslog

# Create the rrd file
# file is located in logdir, and named ip addres of router dot rrd

sub create_file {
        my($fname) = @_;
	if(stat("$logdir"."$fname".".rrd")){
		syslog('warning',"WARNING:filename $fname exists");
	}else {
        RRDs::create (
	"$logdir"."$fname".".rrd",
        "DS:memused:GAUGE:600:U:U",
	"DS:memusedfast:GAUGE:600:U:U",
	"DS:memfree:GAUGE:600:U:U",
	"DS:memfreefast:GAUGE:600:U:U",
	"DS:cpufive:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:600",
        "RRA:AVERAGE:0.5:6:700",
        "RRA:AVERAGE:0.5:24:775",
        "RRA:AVERAGE:0.5:288:797",
        "RRA:MAX:0.5:1:600",
        "RRA:MAX:0.5:6:700",
        "RRA:MAX:0.5:24:775",
        "RRA:MAX:0.5:288:797",
        "RRA:MIN:0.5:1:600",
        "RRA:MIN:0.5:6:700",
        "RRA:MIN:0.5:24:775",
        "RRA:MIN:0.5:288:797",
        "RRA:LAST:0.5:1:600",
        "RRA:LAST:0.5:6:700",
        "RRA:LAST:0.5:24:775",
        "RRA:LAST:0.5:288:797"
        );
	}
        my $ERROR = RRDs::error;
        if ($ERROR){
	syslog('warning',"WARNING::$ERROR")
        }
}
# SNMP section
# .1.3.6.1.4.1.9.9.48.1.1.1.5 - ciscoMemoryPoolUsed
# .1.3.6.1.4.1.9.9.48.1.1.1.6 - ciscoMemoryPoolFree
# .1.3.6.1.4.1.9.2.1.58 - AvgBusy5 

# Collect the information needed via snmp
# and update the rrd files

sub get_info {
	my ($hostname,$community) = @_;
	($session, $error) = Net::SNMP->session(
             -hostname  => shift || $hostname,
             -community => shift || $community,
             -port      => shift || 161
	);
	if (!defined($session)) {
            syslog('warning',"ERROR::%s.\n", $error);
	}
	my ($memused_ptr,$memfree_ptr,$cpuutil_ptr,@a);
	if (!defined($memused_ptr = $session->get_table('.1.3.6.1.4.1.9.9.48.1.1.1.5'))){
		syslog('warning',"ERROR::%s",$session->error());
		$session->close();
	}
	if (!defined($memfree_ptr = $session->get_table('.1.3.6.1.4.1.9.9.48.1.1.1.6'))){
                syslog('warning',"ERROR::%s",$session->error());
                $session->close();
        }
	if (!defined($cpuutil_ptr = $session->get_table('.1.3.6.1.4.1.9.2.1.58'))){
                syslog('warning',"ERROR::%s",$session->error());
                $session->close();
        }
	for(values %$memused_ptr){ push @a,$_;}
	for(values %$memfree_ptr){ push @a,$_;}
	for(values %$cpuutil_ptr){ push @a,$_;}
	for(my $i=0;$i<5;$i++) { $a[$i]=0 if !defined($a[$i]);}
	if (!stat("$logdir"."$hostname".".rrd")){
		create_file($hostname);
		my $utc = time();
		RRDs::update("$logdir"."$hostname".".rrd","$utc:$a[0]:$a[1]:$a[2]:$a[3]:$a[4]");
	} else {
		my $utc = time();
		RRDs::update("$logdir"."$hostname".".rrd","$utc:$a[0]:$a[1]:$a[2]:$a[3]:$a[4]");
	}

}
#Generate graphics

sub graph {
	my($router)=@_;
	my $rrdsource = "$logdir"."$router".".rrd";
	my $target = "$graphdir"."$router";
        my $date = ctime(time());
        RRDs::graph ("$target-memory-day.gif",
        "--title","$router Memory Usage Daily",
        "--start","-86000","-l","0",
        "--vertical-label","Memory Used in Bytes",
        "DEF:memused=$rrdsource:memused:AVERAGE",
        "AREA:memused#333333:Used",
        "GPRINT:memused:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memused:MIN:Min\\:%8.2lf%s",
        "GPRINT:memused:MAX:Max\\:%8.2lf%s",
        "GPRINT:memused:LAST:Current\\:%8.2lf%s\\n",
	"DEF:memfree=$rrdsource:memfree:AVERAGE",
        "STACK:memfree#CCCCCC:Free",
        "GPRINT:memfree:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memfree:MIN:Min\\:%8.2lf%s",
        "GPRINT:memfree:MAX:Max\\:%8.2lf%s",
        "GPRINT:memfree:LAST:Current\\:%8.2lf%s\\n",
        "COMMENT:Generated by cimon http://rousse.pm.org/cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
	syslog('warning',"ERROR::$ERROR");
        }
        RRDs::graph ("$target-cpu-day.gif",
        "--title","$router CPU Utilization Daily ",
        "--start","-86000","-l","0",
        "--vertical-label","Percents","-u","100",
        "DEF:cpu=$rrdsource:cpufive:AVERAGE",
        "AREA:cpu#00CC00:CPU Utilization",
        "GPRINT:cpu:AVERAGE:Avg\\:%2.0lf%s",
        "GPRINT:cpu:MIN:Min\\:%2.0lf%s",
        "GPRINT:cpu:MAX:Max\\:%2.0lf%s",
        "GPRINT:cpu:LAST:Current\\:%2.0lf%s\\n",
        "COMMENT:Generated by cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
	syslog('warning',"ERROR::$ERROR");
        }
	RRDs::graph ("$target-memory-day.gif",
        "--title","$router Memory Usage Daily",
        "--start","-86000","-l","0",
        "--vertical-label","Memory Used in Bytes",
        "DEF:memused=$rrdsource:memused:AVERAGE",
        "AREA:memused#333333:Used",
        "GPRINT:memused:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memused:MIN:Min\\:%8.2lf%s",
        "GPRINT:memused:MAX:Max\\:%8.2lf%s",
        "GPRINT:memused:LAST:Current\\:%8.2lf%s\\n",
        "DEF:memfree=$rrdsource:memfree:AVERAGE",
        "STACK:memfree#CCCCCC:Free",
        "GPRINT:memfree:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memfree:MIN:Min\\:%8.2lf%s",
        "GPRINT:memfree:MAX:Max\\:%8.2lf%s",
        "GPRINT:memfree:LAST:Current\\:%8.2lf%s\\n",
        "COMMENT:Generated by cimon http://rousse.pm.org/cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
        syslog('warning',"ERROR::$ERROR");
        }
        RRDs::graph ("$target-cpu-day.gif",
        "--title","$router CPU Utilization Daily ",
        "--start","-86000","-l","0",
        "--vertical-label","Percents","-u","100",
        "DEF:cpu=$rrdsource:cpufive:AVERAGE",
        "AREA:cpu#00CC00:CPU Utilization",
        "GPRINT:cpu:AVERAGE:Avg\\:%2.0lf%s",
        "GPRINT:cpu:MIN:Min\\:%2.0lf%s",
        "GPRINT:cpu:MAX:Max\\:%2.0lf%s",
        "GPRINT:cpu:LAST:Current\\:%2.0lf%s\\n",
        "COMMENT:Generated by cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
        syslog('warning',"ERROR::$ERROR");
        }
	RRDs::graph ("$target-memory-week.gif",
        "--title","$router Memory Usage Weekly",
        "--start","-604800","-l","0",
        "--vertical-label","Memory Used in Bytes",
        "DEF:memused=$rrdsource:memused:AVERAGE",
        "AREA:memused#333333:Used",
        "GPRINT:memused:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memused:MIN:Min\\:%8.2lf%s",
        "GPRINT:memused:MAX:Max\\:%8.2lf%s",
        "GPRINT:memused:LAST:Current\\:%8.2lf%s\\n",
        "DEF:memfree=$rrdsource:memfree:AVERAGE",
        "STACK:memfree#CCCCCC:Free",
        "GPRINT:memfree:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memfree:MIN:Min\\:%8.2lf%s",
        "GPRINT:memfree:MAX:Max\\:%8.2lf%s",
        "GPRINT:memfree:LAST:Current\\:%8.2lf%s\\n",
        "COMMENT:Generated by cimon http://rousse.pm.org/cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
        syslog('warning',"ERROR::$ERROR");
        }
        RRDs::graph ("$target-cpu-week.gif",
        "--title","$router CPU Utilization Weekly",
        "--start","-604800","-l","0",
        "--vertical-label","Percents","-u","100",
        "DEF:cpu=$rrdsource:cpufive:AVERAGE",
        "AREA:cpu#00CC00:CPU Utilization",
        "GPRINT:cpu:AVERAGE:Avg\\:%2.0lf%s",
        "GPRINT:cpu:MIN:Min\\:%2.0lf%s",
        "GPRINT:cpu:MAX:Max\\:%2.0lf%s",
        "GPRINT:cpu:LAST:Current\\:%2.0lf%s\\n",
        "COMMENT:Generated by cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
        syslog('warning',"ERROR::$ERROR");
        }
	RRDs::graph ("$target-memory-month.gif",
        "--title","$router Memory Usage Monthly",
        "--start","-2600640","-l","0",
        "--vertical-label","Memory Used in Bytes",
        "DEF:memused=$rrdsource:memused:AVERAGE",
        "AREA:memused#333333:Used",
        "GPRINT:memused:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memused:MIN:Min\\:%8.2lf%s",
        "GPRINT:memused:MAX:Max\\:%8.2lf%s",
        "GPRINT:memused:LAST:Current\\:%8.2lf%s\\n",
        "DEF:memfree=$rrdsource:memfree:AVERAGE",
        "STACK:memfree#CCCCCC:Free",
        "GPRINT:memfree:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memfree:MIN:Min\\:%8.2lf%s",
        "GPRINT:memfree:MAX:Max\\:%8.2lf%s",
        "GPRINT:memfree:LAST:Current\\:%8.2lf%s\\n",
        "COMMENT:Generated by cimon http://rousse.pm.org/cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
        syslog('warning',"ERROR::$ERROR");
        }
        RRDs::graph ("$target-cpu-month.gif",
        "--title","$router CPU Utilization Monthly",
        "--start","-2600640","-l","0",
        "--vertical-label","Percents","-u","100",
        "DEF:cpu=$rrdsource:cpufive:AVERAGE",
        "AREA:cpu#00CC00:CPU Utilization",
        "GPRINT:cpu:AVERAGE:Avg\\:%2.0lf%s",
        "GPRINT:cpu:MIN:Min\\:%2.0lf%s",
        "GPRINT:cpu:MAX:Max\\:%2.0lf%s",
        "GPRINT:cpu:LAST:Current\\:%2.0lf%s\\n",
        "COMMENT:Generated by cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
        syslog('warning',"ERROR::$ERROR");
        }
	RRDs::graph ("$target-memory-year.gif",
        "--title","$router Memory Usage Yearly",
        "--start","-31557600","-l","0",
        "--vertical-label","Memory Used in Bytes",
        "DEF:memused=$rrdsource:memused:AVERAGE",
        "AREA:memused#333333:Used",
        "GPRINT:memused:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memused:MIN:Min\\:%8.2lf%s",
        "GPRINT:memused:MAX:Max\\:%8.2lf%s",
        "GPRINT:memused:LAST:Current\\:%8.2lf%s\\n",
        "DEF:memfree=$rrdsource:memfree:AVERAGE",
        "STACK:memfree#CCCCCC:Free",
        "GPRINT:memfree:AVERAGE:Avg\\:%8.2lf%s",
        "GPRINT:memfree:MIN:Min\\:%8.2lf%s",
        "GPRINT:memfree:MAX:Max\\:%8.2lf%s",
        "GPRINT:memfree:LAST:Current\\:%8.2lf%s\\n",
        "COMMENT:Generated by cimon http://rousse.pm.org/cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
        syslog('warning',"ERROR::$ERROR");
        }
        RRDs::graph ("$target-cpu-year.gif",
        "--title","$router CPU Utilization Yearly ",
        "--start","-31557600","-l","0",
        "--vertical-label","Percents","-u","100",
        "DEF:cpu=$rrdsource:cpufive:AVERAGE",
        "AREA:cpu#00CC00:CPU Utilization",
        "GPRINT:cpu:AVERAGE:Avg\\:%2.0lf%s",
        "GPRINT:cpu:MIN:Min\\:%2.0lf%s",
        "GPRINT:cpu:MAX:Max\\:%2.0lf%s",
        "GPRINT:cpu:LAST:Current\\:%2.0lf%s\\n",
        "COMMENT:Generated by cimon\\c",
        "COMMENT:$date\\c"
        );
        if (my $ERROR = RRDs::error) {
        syslog('warning',"ERROR::$ERROR");
        }
}
